#!/bin/bash
## Process *.Rnw files
## Written May 14, 2010
## Taha Ahmed

####################################################
# For now, MAKE SURE that the argument consists of
# a complete filename, with extension, and 
# in the directory of the Rnw file 
####################################################

clear
echo "---------------------------------------------------------------"
echo "cheRTeX -- a script for processing R-Sweave-LaTeX-TikZ projects"
echo "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^"
echo "MMX -- taha.ahmed@mkem.uu.se -- CHEPEC doctoral degree project"
echo "---------------------------------------------------------------"

## If the file .latexmkrc exists in the current directory,
## set the flag ltxmkrc=TRUE
ltxmkrc=false
echo "--- Looking for .latexmkrc"
if [ -e .latexmkrc ]; then
  ltxmkrc=true
   echo "+++ LaTeXMK RC file invoked"
else
   echo "--- This job did not request LaTeXMK RC file"
fi


Rfiletype="R"
TeXfiletype="tex"
tikzfiles="*.tikz"
Rnwfiles="*.Rnw"
auxfiles="*.aux *.bbl *.bcf *.blg *.fdb* *.figlist *.fls *.lof *.log *.lot *.lox *.makefile *.map *.out *.run* *.tdo *.toc *.dep *.dpth"

if [ $# -eq 1 ]; then
	# Check if the argument contains a filetype
	# Assumes a complete filename was passed
	jobfilename=$1
	jobfiletype=${jobfilename#*.} # File extension
	jobname=${jobfilename%.*} # Filename without extension part

	echo "<cheRTeX> Detected filename: " $jobname
	echo "<cheRTeX> Detected extension:" $jobfiletype

	# Verifying that the file extension is *.Rnw
	if [[ $jobfiletype == "Rnw" || $jobfiletype == "rnw" || $jobfiletype == "RNW" ]]; then
		# File extension is *.Rnw
		echo "<cheRTeX> Detected *.Rnw extension"
		jobfiletype="Rnw"
	else
		# File extension is not *.Rnw
		echo "<cheRTeX> File extension should be *.Rnw"
		echo "<cheRTeX> Terminating..."
		exit 1
	fi

	# Introducing a short delay to enable on-screen reading of previous echo
	/home/taha/chepec/chetex/common/bash/simpledelay.sh 2
	echo "Delay completed"

   ## Handle knitr or pgfSweave jobs separately
   ## But how should we tell the difference between them?
   ## There is no obvious way to tell the difference (apart from reading the *.Rnw file)
   ## IN ALL KNITR DIRECTORIES, CREATE A FILE NAMED: .knitme
   # If the file .knitme exists in the current directory,
   # run knitr commands, otherwise run pgfsweave commands
   echo "--- Looking for .knitme"
   if [ -e .knitme ]; then
      # Run knitr commands for this job
      echo "<cheRTeX> -----------------------"
      echo "<cheRTeX> This is a job for knitr"
      echo "<cheRTeX> -----------------------"  
      
      # Knit
	   echo "<cheRTeX> Knitting..."
      Rscript -e "library(knitr); library(methods); knit('$jobname.$jobfiletype')"
      
      # Introduce delay to give time to read Rscript exit status
   	echo "<cheRTeX> -----------------------"
   	echo "<cheRTeX> Rscript knitr completed"
   	echo "<cheRTeX> -----------------------"
   	/home/taha/chepec/chetex/common/bash/simpledelay.sh 4
   else
      # Run pgfSweave commands
      echo "<cheRTeX> ---------------------------"
      echo "<cheRTeX> This is a job for pgfSweave"
      echo "<cheRTeX> ---------------------------"
      
      # Tangle
	   echo "<cheRTeX> Tangling..."
   	R CMD Stangle $jobname.$jobfiletype
	   # Weave
	   echo "<cheRTeX> Weaving..."
   	R CMD pgfsweave --graphics-only $jobname.$jobfiletype
	
	   # Introduce delay to give time to read R CMD exit status
   	echo "<cheRTeX> -------------------------"
   	echo "<cheRTeX> R CMD pgfsweave completed"
   	echo "<cheRTeX> -------------------------"
   	/home/taha/chepec/chetex/common/bash/simpledelay.sh 4
   fi
		
	# Run vc script if vc exists in working directory
	echo "<cheRTeX> Running vc script"
	if [ -f vc ]; then
      ./vc
   fi
	
	# Run pdflatex, bibtex, and company
   if $ltxmkrc; then
      echo "<cheRTeX> Calling LaTeXMK with RC file"
      /home/taha/chepec/chetex/common/bash/simpledelay.sh 2
      latexmk -r .latexmkrc -pdf -bibtex $jobname
   else
	   echo "<cheRTeX> Calling LaTeXMK"
      /home/taha/chepec/chetex/common/bash/simpledelay.sh 2
	   latexmk -pdf -bibtex $jobname
   fi

   ## THIS IS A QUICK-AND-DIRTY FIX FOR SAMPLE-MATRIX
   # If $jobname is sample-matrix, run knitr and markdown
   if [[ $jobname == "sample-matrix" ]]; then
      echo "<cheRTeX> -----------------------------------------------------------"
      echo "<cheRTeX> sample-matrix: update SHINY, run knit() + markdownToHTML()"
      echo "<cheRTeX> -----------------------------------------------------------"
      
      # Old attempts, never worked 
      #PORT=$(</home/taha/chepec/chetex/sample-matrix/shinyapp/shiny.port)
      #bash -c "R -e 'shiny::runApp(\"/home/taha/chepec/chetex/sample-matrix/shinyapp\", port = $PORT)'"
      # Before starting new shiny process, kill any previous instances
      pkill -f "shiny::runApp"
      bash -c "/home/taha/chepec/chetex/common/bash/shiny-matrix.sh" & 
      Rscript -e "library(knitr); knit(input='$jobname.Rmd', output='$jobname.md')"
      Rscript -e "library(markdown); markdownToHTML('$jobname.md', '$jobname.html', stylesheet = 'custom.css')"
   fi

else
	# Either no arguments, or more than one argument
	if [ $# -eq 0 ]; then
		# Zero arguments. Present a menu of choices
		echo "This is cheRTeX POST-PROCESSING" # only one choice for now
		echo "<1> 'pdf-all'  -- Process all .tikz files to pdf graphics"
		echo "<2> 'clean-up' -- Remove all auxiliary files"
		echo "<3> 'wipe-dir' -- Remove all non-essential files and subdirectories"
		echo "Any other input exits the program"
		read usrchoice
		
		## Determine number of .Rnw files in current directory
		#Rnwfileno=$(ls -1 $Rnwfiles | wc -l)
		#echo "No of .Rnw files: $Rnwfileno"
		#
		## Check if number of .Rnw files larger than one
		#if [ $Rnwfileno -gt 1 ]; then
		#	# If larger than one, ask for user input
		#	# Indicates more than one Rnw file in current directory.	
		#	# This introduces a naming ambiguity.
		#	# Resolve by asking user for current jobname
		#	echo "Found $Rnwfileno .Rnw files in current directory"
		#	echo "Please specify the jobname"
		#	read jobname
		#	if [ -z "$jobname" ]; then
		#		# string is null
		#		echo "Specified jobname cannot be parsed. Terminating..."
		#		exit 1
		#	fi
		#else
		#   # There is exactly one *.Rnw file is current directory
		#	# Fetch the jobname from the .Rnw filename by stripping off the file extension
		#	Rnwfilename=$(ls -1 $Rnwfiles)
		#	jobname=${Rnwfilename%.*}
		#fi
		#
		#echo "Jobname set to: $jobname"
		
		if [[ $usrchoice == "pdf-all" || $usrchoice == "1" ]]; then
			echo "<1> 'pdf-all' chosen"
			# This for loop ONLY USED to determine number of *.tikz files in directory
			for tikzfiles in "$tikzfiles"; do tikzfilenumber=${#tikzfiles}; done
			echo "cheRTeX detected $tikzfilenumber TikZ files for processing"
			echo "Starting TikZ file processing..."
			/home/taha/chepec/chetex/common/bash/simpledelay.sh 2
			
			for tikzfilename in $tikzfiles; do
				# Call tikz2pdf 
				echo "<Executing> tikz2pdf $tikzfilename"
				tikz2pdf --once $tikzfilename
			done
			echo "Completed TikZ file processing"
		fi

      if [[ $usrchoice == "clean-up" || $usrchoice == "2" ]]; then
         echo "<2> 'clean-up' chosen"
         rm $auxfiles
         # Still, a rather crude way of cleaning up...
      fi
      
      if [[ $usrchoice == "wipe-dir" || $usrchoice == "3" ]]; then
         echo "<3> 'wipe-dir' chosen"
         ## Remove all but non-essential files
         # get the name of the current directory
         currdirname=${PWD##*/}
         # get a timestamp
         timestamp=$(date +%s)
         # create a unique tmp-dir name
         tmpdirname="${timestamp}-${currdirname}"
         # make a directory in chepec/tmp with the name of the current dir
         mkdir /home/taha/chepec/tmp/$tmpdirname
         # Copy the contents of the current directory to the tmp/$currdirname directory
         cp * -R /home/taha/chepec/tmp/$tmpdirname
         # Empty the current directory of all contents
         rm * -R # note: hidden files and subdir unaffected
         # Return the stuff we want to keep after wipe-dir
         # (we are of course assuming that the following 4 file(type)s always exist)
         # (if in fact they do not exist, use conditional statements instead (see below)
         cp /home/taha/chepec/tmp/$tmpdirname/*.Rnw .
         cp /home/taha/chepec/tmp/$tmpdirname/vc .
         cp /home/taha/chepec/tmp/$tmpdirname/vc.tex .
         cp /home/taha/chepec/tmp/$tmpdirname/vc-git.awk .
         ## Return stuff that may not always exist (check first...)
         ## The use of conditionals is mainly to avoid annoying "file does not exist" messages...
         # Return *.Rproj file (removal is unnecessary and makes RStudio less useful)
         Rprojfiles=`ls -1 /home/taha/chepec/tmp/$tmpdirname/*.Rproj 2>/dev/null | wc -l`
         if [ $Rprojfiles != 0 ]; then 
            cp /home/taha/chepec/tmp/$tmpdirname/*.Rproj .
         fi
         # Return *.rda files (considering peak-data files, which "cost" a lot to create)
         rdafiles=`ls -1 /home/taha/chepec/tmp/$tmpdirname/*.rda 2>/dev/null | wc -l`
         if [ $rdafiles != 0 ]; then 
            cp /home/taha/chepec/tmp/$tmpdirname/*.rda .
         fi
         # Return *.Rmd files (R markdown source files)
         Rmdfiles=`ls -1 /home/taha/chepec/tmp/$tmpdirname/*.Rmd 2>/dev/null | wc -l`
         if [ $Rmdfiles != 0 ]; then 
            cp /home/taha/chepec/tmp/$tmpdirname/*.Rmd .
         fi
         # Return *.css files (css files) [for sample-matrix]
         cssfiles=`ls -1 /home/taha/chepec/tmp/$tmpdirname/*.css 2>/dev/null | wc -l`
         if [ $cssfiles != 0 ]; then 
            cp /home/taha/chepec/tmp/$tmpdirname/*.css .
         fi
         # Return .knitme file [empty file used to indicate knitr jobs]
         knitmefile=`ls -1 /home/taha/chepec/tmp/$tmpdirname/.knitme 2>/dev/null | wc -l`
         if [ $knitmefile != 0 ]; then 
            cp /home/taha/chepec/tmp/$tmpdirname/.knitme .
         fi
      fi

		echo "Terminating..."
		
		exit 1
	fi
	
	## Here is the wild land of more than one *.Rnw file in current directory
	
	echo "<cheRTeX> This script can be run with one argument is process mode,"
	echo "<cheRTeX> or with zero arguments in post-processing mode."
	echo "<cheRTeX> Terminating..."
	exit 1
fi


echo "------------------------------------"
echo "===  sample-matrix.sh completed  ==="
echo "=== $(date) ==="
echo "------------------------------------"
/home/taha/chepec/chetex/common/bash/simpledelay.sh 10

exit 1
